<?php

namespace EventPortal\Domain\Agenda\Models;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Support\Carbon;

class Session extends Model
{
    const KEY = 'session';

    /** @var array<string> */
    protected $fillable = [
        'pathway_id',
        'name',
        'description',
        'location',
        'venue_id',
        'starting_at',
        'ending_at',
    ];

    /** @var array<string> */
    protected $casts = [
        'starting_at' => 'datetime:h:i',
        'ending_at' => 'datetime:h:i',
    ];

    public function pathway(): BelongsTo
    {
        return $this->belongsTo(Pathway::class);
    }

    public function scopeForPathway(Builder $query, Pathway $pathway): Builder
    {
        return $query->where('pathway_id', $pathway->id);
    }

    /**
     * @param string|Carbon $time
     */
    public function scopePartiallyOverlaps(Builder $query, $time): Builder
    {
        if ($time instanceof Carbon === false) {
            $time = Carbon::createFromTimeString($time);
        }

        return $query
            ->whereTime('starting_at', '<', $time)
            ->whereTime('ending_at', '>', $time);
    }

    /**
     * @param string|Carbon $startTime
     * @param string|Carbon $endTime
     */
    public function scopeCompletelyOverlaps(Builder $query, $startTime, $endTime): Builder
    {
        if ($startTime instanceof Carbon === false) {
            $startTime = Carbon::createFromTimeString($startTime);
        }

        if ($endTime instanceof Carbon === false) {
            $endTime = Carbon::createFromTimeString($endTime);
        }

        return $query->where(function ($query) use ($endTime, $startTime) {
            return $query
                ->where(function (Builder $query) use ($startTime, $endTime) {
                    return $query
                        ->where(function (Builder $query) use ($startTime, $endTime) {
                            return $query
                                ->whereTime('starting_at', '>=', $startTime)
                                ->whereTime('ending_at', '<=', $startTime);
                        })
                        ->orWhere(function (Builder $query) use ($startTime, $endTime) {
                            return $query
                                ->whereTime('starting_at', '<=', $startTime)
                                ->whereTime('starting_at', '>=', $endTime);
                        });
                })
                ->orWhere('starting_at', '>=', $startTime)
                ->where('ending_at', '<=', $endTime);
        });
    }
}
