<?php

namespace EventPortal\Domain\Table\Factories;

use EventPortal\Domain\Component\Filter\Filter;
use EventPortal\Domain\Table\Columns;
use EventPortal\Domain\Table\DataObjects\ColumnSelector;
use EventPortal\Domain\Table\DataObjects\FilterField;
use EventPortal\Domain\Table\DataObjects\SimpleColumn;
use EventPortal\Domain\Table\Table;
use Illuminate\Contracts\Pagination\LengthAwarePaginator;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Illuminate\Support\Collection;
use Illuminate\Support\Str;

class TableFactory
{
   /** @var Model */
    protected $model;

    /** @var array<Model> */
    protected $context = [];

    /** @var Collection */
    protected $availableColumns;

    /** @var Collection */
    protected $columns;

    /** @var Collection */
    protected $availableFilters;

    /** @var Collection */
    protected $filters;

    /** @var Collection */
    protected $globalActions;

    /** @var \Closure */
    protected $action;

    /** @var int */
    protected $page = 1;

    /** @var int */
    protected $size = 5;

    /** @var string */
    protected $searchQuery;

    /** @var Builder */
    protected $query;

    /** @var callable */
    protected $tappedQuery;

    /** @var array */
    protected $joins = [];

    public function __construct()
    {
        $this->filters = collect();
        $this->availableFilters = collect();
        $this->columns = collect();
        $this->availableColumns = collect();
        $this->globalActions = collect();
    }

    public function model(Model $model): self
    {
        $this->model = $model;

        if ($this->query === null) {
            $this->query = $model->newQuery();
        }

        return $this;
    }

    /**
     * @param mixed ...$models
     */
    public function context(...$models): self
    {
        $keyedModels = [];

        foreach ($models as $model) {
            $keyedModels[$model::KEY] = $model;
        }

        $this->context = $keyedModels;

        return $this;
    }

    public function setColumns(callable $columns): self
    {
        $this->availableColumns = $columns;

        return $this;
    }

    public function setFilters(callable $filters): self
    {
        $this->availableFilters = $filters;

        return $this;
    }

    public function setRowAction(?callable $action): self
    {
        if ($action !== null) {
            $this->action = $action;
        }

        return $this;
    }

    public function addGlobalAction(callable $action): self
    {
        $this->globalActions = $action;

        return $this;
    }

    public function request(Request $request): self
    {
        $this
            ->columns($request->get('cols'))
            ->size($request->get('size'))
            ->page($request->get('page'))
            ->search(request('query'))
            ->filters($request->get('filter'));

        return $this;
    }

    public function columns(?string $selectedColumns): self
    {
        if ($selectedColumns !== null) {
            session()->put(
                $this->getColumnSessionKey(),
                Str::of($selectedColumns)->explode(',')->toArray()
            );
        }

        return $this;
    }

    public function filters(?array $filters): self
    {
        if ($filters !== null) {
            $this->filters = collect($filters)->filter();
        }

        return $this;
    }

    public function page(?string $page): self
    {
        $this->page = $page;

        return $this;
    }

    public function size(?string $size): self
    {
        if ($size !== null) {
            $this->size = $size;
        }

        return $this;
    }

    public function search(?string $searchQuery): self
    {
        $this->searchQuery = $searchQuery;

        return $this;
    }

    public function join(string $table, callable $joinCallback): self
    {
        $this->joins[$table] = $joinCallback;

        return $this;
    }

    public function query(Builder $query): self
    {
        $this->query = $query;

        if ($this->model === null && $query instanceof Builder) {
            $this->model($query->getModel());
        }

        return $this;
    }

    public function tapQuery(callable $query): self
    {
        $this->tappedQuery = $query;

        return $this;
    }

    public function get(): Table
    {
        $this->resolve('globalActions');
        $this->resolve('availableFilters');
        $this->resolve('availableColumns');

        $columnsToDisplay = $this->getColumnsToDisplay();

        $this->setAvailableFiltersBasedOnColumns($columnsToDisplay);

        return Table::make()
            ->currentFilters($this->filters)
            ->currentQuery($this->searchQuery)
            ->defaultPageSize($this->size)
            ->withColumns($columnsToDisplay->mapInto(SimpleColumn::class))
            ->withAllColumns($this->availableColumns->mapInto(ColumnSelector::class))
            ->withAction($this->action)
            ->withGlobalActions($this->globalActions)
            ->withFiltersFields($this->availableFilters->mapInto(FilterField::class))
            ->withContext($this->context)
            ->withData($this->getData($columnsToDisplay), $columnsToDisplay);
    }

    private function getData(Collection $columns): LengthAwarePaginator
    {
        $columns = Columns::make($columns);

        return $this
            ->query
            ->select($columns->forSelect($this->model))
            ->when($this->tappedQuery, function (Builder $query): Builder {
                return tap($query, function ($query): Builder {
                    return app()->call($this->tappedQuery, ['query' => $query]);
                });
            })
            ->when($this->searchQuery, function (Builder $query) use ($columns): Builder {
                return $query->fieldsLike($columns->forSearch(), $this->searchQuery);
            })
            ->when($this->filters, function (Builder $query): Builder {
                return $this->applyFilters($query);
            })
            ->groupBy($this->model->getQualifiedKeyName())
            ->paginate($this->size);
    }

    /**
     * This needs to be refactored, but for now the stupid computer thinks this is documentation.
     */
    private function applyFilters(Builder $query): Builder
    {
        foreach ($this->filters as $fieldName => $value) {
            $filter = $this->findFilterByName($fieldName);

            if ($filter === null || $filter->hasManualLookup() === false) {
                $query->whereFilter($fieldName, $value);

                continue;
            }

            $filter->applyLookup($query, $value, $fieldName);
        }

        return $query;
    }

    private function resolve(string $whatToResolve): Collection
    {
        if ($this->$whatToResolve instanceof Collection && $this->$whatToResolve->isEmpty()) {
            return $this->$whatToResolve;
        }

        $this->$whatToResolve = collect(
            app()->call($this->$whatToResolve, $this->getContext())
        )->filter()->values();

        return $this->$whatToResolve;
    }

    private function findFilterByName(string $fieldName): ?Filter
    {
        return $this->availableFilters->firstWhere('key', $fieldName);
    }

    private function getContext(): array
    {
        return array_merge($this->context, [$this->model::KEY => $this->model]);
    }

    private function getColumnSessionKey(): string
    {
        return "table-{$this->model->getTable()}-columns";
    }

    private function getColumnsToDisplay(): Collection
    {
        $columnKey = $this->getColumnSessionKey();

        if (session($columnKey) !== null) {
            return $this->availableColumns->whereIn('name', session($columnKey));
        }

        return $this->availableColumns->filter->isDefault();
    }

    private function setAvailableFiltersBasedOnColumns(Collection $columns): void
    {
        $this->availableFilters = $this->availableFilters->filter(function (Filter $filter) use ($columns) {
            return $columns->contains('name', $filter->relatedColumn);
        });
    }
}
