<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Permission\Models\Permission;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Tests\Support\Traits\InteractsWithTeams;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Spatie\Snapshots\MatchesSnapshots;

class EventIndexTest extends TestCase
{
    use DatabaseMigrations, MatchesSnapshots, InteractsWithTeams, SeedRolesAndPermissions;

    /** @test **/
    public function can_view_index()
    {
        factory(Event::class)->create(['name' => 'Hello World!']);
        $admin = factory(Admin::class)->create();
        $permission = factory(Permission::class)->create([
            'name' => 'event.index',
        ]);
        $admin->givePermissionTo($permission);

        $this->assertMatchesJsonSnapshot($this->actingAs($admin->user)->get(route('admin.event.index'))->props());
    }

    /** @test **/
    public function shows_index_with_columns()
    {
        factory(Event::class)->create();
        $admin = factory(Admin::class)->create();
        $permission = factory(Permission::class)->create([
            'name' => 'event.index',
        ]);

        $admin->givePermissionTo($permission);

        $this->actingAs($admin->user)
            ->get(route('admin.event.index') . '?cols=name')
            ->assertSessionHas('table-events-columns', [
                'name',
            ]);
    }

    /** @test **/
    public function shows_index_with_columns_session_doesnt_get_overriden_frist_time_load()
    {
        factory(Event::class)->create();
        $admin = factory(Admin::class)->create();
        $permission = factory(Permission::class)->create([
            'name' => 'event.index',
        ]);

        $admin->givePermissionTo($permission);
        $this->actingAs($admin->user);

        $this->get(route('admin.event.index') . '?cols=name')
            ->assertSessionHas('table-events-columns', [
                'name',
            ]);

        $this->get(route('admin.event.index'))
            ->assertSessionHas('table-events-columns', [
                'name',
            ]);
    }

    /** @test **/
    public function admins_who_are_in_a_team_can_view_index_and_not_see_events_for_other_teams(): void
    {
        $team = factory(Team::class)->create();
        $adminTeam = factory(Team::class)->create();
        $admin = factory(Admin::class)->state('admin')->create();
        $event = factory(Event::class)->create(['name' => 'SHOULD NOT SEE']);
        $adminEvent = factory(Event::class)->create(['name' => 'SHOULD SEE']);

        $this->addToSameTeam([$admin, $adminEvent], $adminTeam);
        $this->addToSameTeam([$event], $team);

        $this->actingAsAdmin($admin);
        $this->assertMatchesJsonSnapshot($this->get(route('admin.event.index'))->props());
    }

    /** @test **/
    public function super_admins_who_are_in_a_team_can_view_index_and_see_all_events(): void
    {
        $team = factory(Team::class)->create();
        $superAdminTeam = factory(Team::class)->create();
        $superAdmin = factory(Admin::class)->state('super-admin')->create();
        $event = factory(Event::class)->create(['name' => 'SHOULD SEE']);
        $superAdminEvent = factory(Event::class)->create(['name' => 'SHOULD SEE']);

        $this->addToSameTeam([$superAdmin, $superAdminEvent], $superAdminTeam);
        $this->addToSameTeam([$event], $team);

        $this->actingAsSuperAdmin($superAdmin);
        $this->assertMatchesJsonSnapshot($this->get(route('admin.event.index'))->props());
    }
}
