<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Permission\Models\Permission;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Domain\Util\Sidebar\Link;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class SidebarTest extends TestCase
{
    use MatchesSnapshots, RefreshDatabase;

    /** @test */
    public function the_primary_sidebar_can_be_generated()
    {
        $sidebar = Sidebar::make('primary')
            ->render();

        $this->assertMatchesJsonSnapshot($sidebar);
    }

    /** @test */
    public function the_sidebar_can_be_generated_for_an_event()
    {
        $event = factory(Event::class)->create();

        $sidebar = Sidebar::make('event.show')
            ->model($event)
            ->render();

        $this->assertMatchesJsonSnapshot($sidebar);
    }

    /** @test */
    public function the_title_can_be_overridden()
    {
        $event = factory(Event::class)->create();

        $sidebar = Sidebar::make('event.show')
            ->model($event)
            ->title('Override')
            ->render();

        $this->assertEquals('Override', $sidebar['title']);
    }

    /** @test **/
    public function the_model_is_injected_into_the_routes()
    {
        $event = factory(Event::class)->create();
        $sidebar = Sidebar::make('test.sidebar.model.injection')
            ->model($event)
            ->addLinks(function (Event $eventForSidebar) {
                return [
                    Link::make('Test Link')
                        ->route('admin.event.show', [
                            'event' => $eventForSidebar->id,
                        ]),
                ];
            })
            ->render();

        $this->assertEquals('admin.event.show', $sidebar['children'][0]['route']);
        $this->assertEquals($event->id, $sidebar['children'][0]['routeParams']['event']);
    }

    /** @test **/
    public function the_sidebar_hides_links_if_user_doesnt_have_permission()
    {
        factory(Permission::class)->create([
            'name' => 'venue.show',
        ]);

        factory(Permission::class)->create([
            'name' => 'agenda.show',
        ]);

        $admin = factory(Admin::class)->create();
        $admin->user->givePermissionTo('venue.show');

        $sidebar = Sidebar::make('test.sidebar.permissions')
            ->user($admin->user)
            ->addLinks(function () {
                return [
                    Link::make('Venue')
                        ->permission('venue.show'),
                    Link::make('Agenda')
                        ->permission('agenda.show'),
                ];
            })
            ->render();

        $links = collect($sidebar['children']);

        $this->assertCount(1, $links->where('name', 'Venue'));
        $this->assertCount(0, $links->where('name', 'Agenda'));
    }
}
